#!/bin/ksh
#
# Copyright (c) 2001 by Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)order_patches.ksh	1.3	01/08/23 SMI"

# This script provides a legal install order for a set of patches in a patch
# directory.
#
# Note: Patch revisions are not considered in the ordering. It is assumed
# that users will not include multiple revisions of a patch in a given patch set.
#
# This algorithm generates a backout list first then reverses the order of the 
# list to get an install list.
#
# This script expects one argument, the patch directory. It returns an ordered
# patch list to stdout.
#

if [ $# != 1 ]; then
   echo "Usage: $0 patch_directory"
   exit 1
fi

SED="/usr/bin/sed"
CAT="/usr/bin/cat"

BACKOUT_FILE="/tmp/Backout.$$"
REQUIRED_FILE="/tmp/Requires.$$"
SCAN_FILE="/tmp/Scan.$$"
TEMP_FILE="/tmp/Junk.$$"

# We'll delete everything in $CLEANUP when we're done.
CLEANUP="$BACKOUT_FILE $REQUIRED_FILE $SCAN_FILE $TEMP_FILE"

# Make sure all files are empty.

for file in $CLEANUP; do
   cp /dev/null $file
done
  
# Look for the string "SUNW_REQUIRES" in all pkginfo files.
# Build a temporary file, SCAN_FILE, with the following line format:
#
#   <patch_id> ( <required_patch> <required_patch> ... )
#
# It's OK for the parentheses to be empty.

if [ ! -d $1 ]; then
   echo "$1 is not a directory."
   exit 2
fi

cd $1 # the patch directory

for patch in `ls -d *`; do

   grep "SUNW_REQUIRES=1" $patch/*/pkginfo | head -1 > $TEMP_FILE # comment close is workaround for bug #4398335 */
   req=`cat $TEMP_FILE`
   if [ "$req" = "" ]; then   # this patch has no SUNW_REQUIRES. Create entry 
                              # in SCAN_FILE in case someone requires it
      echo $patch "( )" >> $SCAN_FILE
   else                       # create the proper requires line
      echo $patch "(" `echo $req | awk -F= '{print $2}'` ")" >> $SCAN_FILE
   fi
done
 
not_done=0

# Scan the patch list, SCAN_FILE.
#    Move patches from SCAN_FILE to REQUIRED_FILE if other patches require them.
#    Move patches from SCAN_FILE to BACKOUT_FILE if no other patches require them.
# When we're done scanning SCAN_FILE, move REQUIRED_FILE back to SCAN_FILE
#   and do the scan again. 
# Keep scanning until REQUIRED_FILE is empty.

while [ $not_done = 0 ]
do
   if [ -s $SCAN_FILE ]; then
      cat $SCAN_FILE | while read patchinfo
      do
         patchnum=`echo $patchinfo | awk '{print $1}'`
         patchbase=${patchnum%-*}
         grep -v "^$patchbase" $SCAN_FILE | grep "$patchbase" > /dev/null
         if [ $? = 0 ]; then
            echo $patchinfo >> $REQUIRED_FILE
         else
            echo $patchnum >> $BACKOUT_FILE
         fi
      done
   fi
   if [ -s $REQUIRED_FILE ]; then
      mv $REQUIRED_FILE $SCAN_FILE
   else
      not_done=1          # We are done!
   fi
done

# Now we reverse the order of the lines in BACKOUT_FILE
#
$CAT -n $BACKOUT_FILE | sort -rn | $SED 's/^[ ]*[0-9]*[	]//' 

# Clean up temporary files.
rm -f $CLEANUP

